@@ -18,7 +18,6 @@ char rcsId_vmware[] =
 
 #include "xf86.h"
 #include "xf86_OSproc.h"
-#include "xf86_ansic.h"
 #include "xf86Resources.h"
 
 #include "compiler.h"	/* inb/outb */
@@ -38,6 +37,26 @@ char rcsId_vmware[] =
 #include "vmware.h"
 #include "guest_os.h"
 #include "vm_device_version.h"
+#include "svga_modes.h"
+
+#ifdef HaveDriverFuncs
+#define VMWARE_DRIVER_FUNC HaveDriverFuncs
+#else
+#define VMWARE_DRIVER_FUNC 0
+#endif
+
+/*
+ * So that the file compiles unmodified when dropped in to a < 6.9 source tree.
+ */
+#ifndef _X_EXPORT
+#define _X_EXPORT
+#endif
+/*
+ * So that the file compiles unmodified when dropped into an xfree source tree.
+ */
+#ifndef XORG_VERSION_CURRENT
+#define XORG_VERSION_CURRENT XF86_VERSION_CURRENT
+#endif
 
 /*
  * Sanity check that xf86PciInfo.h has the correct values (which come from
@@ -63,13 +82,14 @@ char rcsId_vmware[] =
 #define VMWARE_NAME "VMWARE"
 #define VMWARE_DRIVER_NAME "vmware"
 #define VMWARE_MAJOR_VERSION	10
-#define VMWARE_MINOR_VERSION	11
+#define VMWARE_MINOR_VERSION	15
 #define VMWARE_PATCHLEVEL	0
-#define VERSION (VMWARE_MAJOR_VERSION * 65536 + VMWARE_MINOR_VERSION * 256 + VMWARE_PATCHLEVEL)
+#define VMWARE_DRIVER_VERSION \
+   (VMWARE_MAJOR_VERSION * 65536 + VMWARE_MINOR_VERSION * 256 + VMWARE_PATCHLEVEL)
 
 static const char VMWAREBuildStr[] = "VMware Guest X Server " 
     VMW_STRING(VMWARE_MAJOR_VERSION) "." VMW_STRING(VMWARE_MINOR_VERSION)
-    "." VMW_STRING(VMWARE_PATCHLEVEL) " - build=$Name:  $\n";
+    "." VMW_STRING(VMWARE_PATCHLEVEL) " - build=$Name$\n";
 
 static SymTabRec VMWAREChipsets[] = {
     { PCI_CHIP_VMWARE0405, "vmware0405" },
@@ -145,15 +165,32 @@ static XF86ModuleVersionInfo vmwareVersRec = {
 
 typedef enum {
     OPTION_HW_CURSOR,
-    OPTION_NOACCEL
+    OPTION_XINERAMA,
+    OPTION_STATIC_XINERAMA
 } VMWAREOpts;
 
 static const OptionInfoRec VMWAREOptions[] = {
     { OPTION_HW_CURSOR, "HWcursor",     OPTV_BOOLEAN,   {0},    FALSE },
-    { OPTION_NOACCEL,   "NoAccel",      OPTV_BOOLEAN,   {0},    FALSE },
+    { OPTION_XINERAMA,  "Xinerama",     OPTV_BOOLEAN,   {0},    FALSE },
+    { OPTION_STATIC_XINERAMA, "StaticXinerama", OPTV_STRING, {0}, FALSE },
     { -1,               NULL,           OPTV_NONE,      {0},    FALSE }
 };
 
+/* Table of default modes to always add to the mode list. */
+
+typedef struct {
+   int width;
+   int height;
+} VMWAREDefaultMode;
+
+#define SVGA_DEFAULT_MODE(width, height) { width, height, },
+
+static const VMWAREDefaultMode VMWAREDefaultModes[] = {
+   SVGA_DEFAULT_MODES
+};
+
+#undef SVGA_DEFAULT_MODE
+
 static void VMWAREStopFIFO(ScrnInfoPtr pScrn);
 static void VMWARESave(ScrnInfoPtr pScrn);
 
@@ -257,17 +294,6 @@ vmwareSendSVGACmdUpdateFullScreen(VMWAREPtr pVMWARE)
     vmwareSendSVGACmdUpdate(pVMWARE, &BB);
 }
 
-static void
-vmwareSendSVGACmdPitchLock(VMWAREPtr pVMWARE, unsigned long fbPitch)
-{
-   CARD32 *vmwareFIFO = pVMWARE->vmwareFIFO;
-
-   if (pVMWARE->canPitchLock && vmwareFIFO[SVGA_FIFO_MIN] >=
-                                (vmwareReadReg(pVMWARE, SVGA_REG_MEM_REGS) << 2)) {
-      vmwareFIFO[SVGA_FIFO_PITCHLOCK] = fbPitch;
-   }
-}
-
 static CARD32
 vmwareCalculateWeight(CARD32 mask)
 {
@@ -342,7 +368,7 @@ VMXGetVMwareSvgaId(VMWAREPtr pVMWARE)
  *
  *  RewriteTagString --
  *
- *      Rewrites the given string, removing the $Name:  $, and
+ *      Rewrites the given string, removing the $Name$, and
  *      replacing it with the contents.  The output string must
  *      have enough room, or else.
  *
@@ -395,6 +421,129 @@ VMWAREAvailableOptions(int chipid, int busid)
     return VMWAREOptions;
 }
 
+static int
+VMWAREParseTopologyElement(ScrnInfoPtr pScrn,
+                           unsigned int output,
+                           const char *elementName,
+                           const char *element,
+                           const char *expectedTerminators,
+                           Bool needTerminator,
+                           unsigned int *outValue)
+{
+   char buf[10] = {0, };
+   size_t i = 0;
+   int retVal = -1;
+   const char *str = element;
+
+   for (i = 0; str[i] >= '0' && str[i] <= '9'; i++);
+   if (i == 0) {
+      xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Output %u: unable to parse %s.\n",
+                 output, elementName);
+      goto exit;
+   }
+
+   strncpy(buf, str, i);
+   *outValue = atoi(buf);
+
+   if (*outValue > (unsigned short)-1) {
+      xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Output %u: %s must be less than %hu.\n",
+                 output, elementName, (unsigned short)-1);
+      goto exit;
+   }
+
+   str += i;
+
+   if (needTerminator || str[0] != '\0') {
+      Bool unexpected = TRUE;
+
+      for (i = 0; i < strlen(expectedTerminators); i++) {
+         if (str[0] == expectedTerminators[i]) {
+            unexpected = FALSE;
+         }
+      }
+
+      if (unexpected) {
+         xf86DrvMsg(pScrn->scrnIndex, X_INFO,
+                    "Output %u: unexpected character '%c' after %s.\n",
+                    output, str[0], elementName);
+         goto exit;
+      } else {
+         str++;
+      }
+   }
+
+   retVal = str - element;
+
+ exit:
+   return retVal;
+}
+
+static xXineramaScreenInfo *
+VMWAREParseTopologyString(ScrnInfoPtr pScrn,
+                          const char *topology,
+                          unsigned int *retNumOutputs)
+{
+   xXineramaScreenInfo *extents = NULL;
+   unsigned int numOutputs = 0;
+   const char *str = topology;
+
+   xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Parsing static Xinerama topology: Starting...\n");
+
+   do {
+      unsigned int x, y, width, height;
+      int i;
+
+      i = VMWAREParseTopologyElement(pScrn, numOutputs, "width", str, "xX", TRUE, &width);
+      if (i == -1) {
+         goto error;
+      }
+      str += i;
+
+      i = VMWAREParseTopologyElement(pScrn, numOutputs, "height", str, "+", TRUE, &height);
+      if (i == -1) {
+         goto error;
+      }
+      str += i;
+
+      i= VMWAREParseTopologyElement(pScrn, numOutputs, "X offset", str, "+", TRUE, &x);
+      if (i == -1) {
+         goto error;
+      }
+      str += i;
+
+      i = VMWAREParseTopologyElement(pScrn, numOutputs, "Y offset", str, ";", FALSE, &y);
+      if (i == -1) {
+         goto error;
+      }
+      str += i;
+
+      xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Output %u: %ux%u+%u+%u\n",
+                 numOutputs, width, height, x, y);
+
+      numOutputs++;
+      extents = xrealloc(extents, numOutputs * sizeof (xXineramaScreenInfo));
+      extents[numOutputs - 1].x_org = x;
+      extents[numOutputs - 1].y_org = y;
+      extents[numOutputs - 1].width = width;
+      extents[numOutputs - 1].height = height;
+   } while (*str != 0);
+
+   xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Parsing static Xinerama topology: Succeeded.\n");
+   goto exit;
+
+ error:
+   xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Parsing static Xinerama topology: Failed.\n");
+
+   xfree(extents);
+   extents = NULL;
+   numOutputs = 0;
+
+ exit:
+   *retNumOutputs = numOutputs;
+   return extents;
+}
+
+
 static Bool
 VMWAREPreInit(ScrnInfoPtr pScrn, int flags)
 {
@@ -406,6 +555,7 @@ VMWAREPreInit(ScrnInfoPtr pScrn, int flags)
     int i;
     ClockRange* clockRanges;
     IOADDRESS domainIOBase = 0;
+    Bool useXinerama = TRUE;
 
 #ifndef BUILD_FOR_420
     domainIOBase = pScrn->domainIOBase;
@@ -710,14 +860,33 @@ VMWAREPreInit(ScrnInfoPtr pScrn, int flags)
     }
     xf86DrvMsg(pScrn->scrnIndex, from, "Using %s cursor\n",
                pVMWARE->hwCursor ? "HW" : "SW");
-    if (xf86IsOptionSet(options, OPTION_NOACCEL)) {
-        pVMWARE->noAccel = TRUE;
-        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Acceleration disabled\n");
-    } else {
-        pVMWARE->noAccel = FALSE;
-    }
     pScrn->videoRam = pVMWARE->videoRam / 1024;
     pScrn->memPhysBase = pVMWARE->memPhysBase;
+
+    /*
+     * Init xinerama preferences.
+     */
+    useXinerama = xf86ReturnOptValBool(options, OPTION_XINERAMA,
+                                       pVMWARE->vmwareCapability & SVGA_CAP_MULTIMON);
+    if (useXinerama && !(pVMWARE->vmwareCapability & SVGA_CAP_MULTIMON)) {
+       xf86DrvMsg(pScrn->scrnIndex, X_WARNING,
+                  "Xinerama is not safely supported by the current virtual hardware. "
+                  "Do not request resolutions that require > 16MB of framebuffer.\n");
+    }
+
+
+    if (useXinerama && xf86IsOptionSet(options, OPTION_STATIC_XINERAMA)) {
+       char *topology = xf86GetOptValString(options, OPTION_STATIC_XINERAMA);
+       if (topology) {
+          pVMWARE->xineramaState =
+             VMWAREParseTopologyString(pScrn, topology, &pVMWARE->xineramaNumOutputs);
+
+         pVMWARE->xineramaStatic = pVMWARE->xineramaState != NULL;
+
+         xfree(topology);
+       }
+    }
+
     xfree(options);
 
     {
@@ -782,12 +951,18 @@ VMWAREPreInit(ScrnInfoPtr pScrn, int flags)
         xf86LoaderReqSymLists(ramdacSymbols, NULL);
     }
 
-    if (!pVMWARE->noAccel) {
-        if (!xf86LoadSubModule(pScrn, "xaa")) {
-            VMWAREFreeRec(pScrn);
-            return FALSE;
-        }
-        xf86LoaderReqSymLists(vmwareXaaSymbols, NULL);
+    /* Initialise VMWARE_CTRL extension. */
+    VMwareCtrl_ExtInit(pScrn);
+
+    /* Initialise Xinerama extension. */
+    if (useXinerama) {
+       VMwareXinerama_ExtInit(pScrn);
+    }
+
+    if (pVMWARE->xinerama && pVMWARE->xineramaStatic) {
+       xf86DrvMsg(pScrn->scrnIndex, X_INFO, pVMWARE->xineramaState ?
+                                            "Using static Xinerama.\n" :
+                                            "Failed to configure static Xinerama.\n");
     }
 
     return TRUE;
@@ -909,7 +1084,7 @@ VMWARERestore(ScrnInfoPtr pScrn)
 }
 
 static Bool
-VMWAREModeInit(ScrnInfoPtr pScrn, DisplayModePtr mode)
+VMWAREModeInit(ScrnInfoPtr pScrn, DisplayModePtr mode, Bool rebuildPixmap)
 {
     vgaHWPtr hwp = VGAHWPTR(pScrn);
     vgaRegPtr vgaReg = &hwp->ModeReg;
@@ -948,6 +1123,20 @@ VMWAREModeInit(ScrnInfoPtr pScrn, DisplayModePtr mode)
     VmwareLog(("fbSize:        %u\n", pVMWARE->FbSize));
     VmwareLog(("New dispWidth: %u\n", pScrn->displayWidth));
 
+    if (rebuildPixmap) {
+        pScrn->pScreen->ModifyPixmapHeader((*pScrn->pScreen->GetScreenPixmap)(pScrn->pScreen),
+                                           pScrn->pScreen->width,
+                                           pScrn->pScreen->height,
+                                           pScrn->pScreen->rootDepth,
+                                           pScrn->bitsPerPixel,
+                                           PixmapBytePad(pScrn->displayWidth,
+                                                         pScrn->pScreen->rootDepth),
+                                           (pointer)(pVMWARE->FbBase + pScrn->fbOffset));
+
+        (*pScrn->EnableDisableFBAccess)(pScrn->pScreen->myNum, FALSE);
+        (*pScrn->EnableDisableFBAccess)(pScrn->pScreen->myNum, TRUE);
+    }
+
     vgaHWProtect(pScrn, FALSE);
 
     /*
@@ -956,12 +1145,30 @@ VMWAREModeInit(ScrnInfoPtr pScrn, DisplayModePtr mode)
      */
 
     /*
-     * Let XAA know about the mode change.
+     * Update Xinerama info appropriately.
      */
-    if (!pVMWARE->noAccel) {
-        if (!vmwareXAAModeInit(pScrn, mode)) {
-            return FALSE;
-        }
+    if (pVMWARE->xinerama && !pVMWARE->xineramaStatic) {
+       if (pVMWARE->xineramaNextState) {
+          xfree(pVMWARE->xineramaState);
+          pVMWARE->xineramaState = pVMWARE->xineramaNextState;
+          pVMWARE->xineramaNumOutputs = pVMWARE->xineramaNextNumOutputs;
+
+          pVMWARE->xineramaNextState = NULL;
+          pVMWARE->xineramaNextNumOutputs = 0;
+       } else {
+          VMWAREXineramaPtr basicState =
+             (VMWAREXineramaPtr)xcalloc(1, sizeof (VMWAREXineramaRec));
+          if (basicState) {
+             basicState->x_org = 0;
+             basicState->y_org = 0;
+             basicState->width = vmwareReg->svga_reg_width;
+             basicState->height = vmwareReg->svga_reg_height;
+
+             xfree(pVMWARE->xineramaState);
+             pVMWARE->xineramaState = basicState;
+             pVMWARE->xineramaNumOutputs = 1;
+          }
+       }
     }
 
     return TRUE;
@@ -999,9 +1206,6 @@ VMWAREInitFIFO(ScrnInfoPtr pScrn)
     vmwareFIFO[SVGA_FIFO_NEXT_CMD] = min * sizeof(CARD32);
     vmwareFIFO[SVGA_FIFO_STOP] = min * sizeof(CARD32);
     vmwareWriteReg(pVMWARE, SVGA_REG_CONFIG_DONE, 1);
-
-    pVMWARE->canPitchLock =
-        extendedFifo && (vmwareFIFO[SVGA_FIFO_CAPABILITIES] & SVGA_FIFO_CAP_PITCHLOCK);
 }
 
 static void
@@ -1029,12 +1233,6 @@ VMWARECloseScreen(int scrnIndex, ScreenPtr pScreen)
             vmwareCursorCloseScreen(pScreen);
         }
 
-        if (pVMWARE->xaaInfo) {
-            vmwareXAACloseScreen(pScreen);
-        }
-
-        vmwareSendSVGACmdPitchLock(pVMWARE, 0);
-
         VMWARERestore(pScrn);
         VMWAREUnmapMem(pScrn);
 
@@ -1044,6 +1242,10 @@ VMWARECloseScreen(int scrnIndex, ScreenPtr pScreen)
     pScreen->CloseScreen = save->CloseScreen;
     pScreen->SaveScreen = save->SaveScreen;
 
+#if VMWARE_DRIVER_FUNC
+    pScrn->DriverFunc = NULL;
+#endif
+
     return (*pScreen->CloseScreen)(scrnIndex, pScreen);
 }
 
@@ -1084,8 +1286,10 @@ VMWAREPreDirtyBBUpdate(ScrnInfoPtr pScrn, int nboxes, BoxPtr boxPtr)
      */
     while (nboxes--) {
         if (BOX_INTERSECT(*boxPtr, pVMWARE->hwcur.box)) {
-	    PRE_OP_HIDE_CURSOR();
-	    pVMWARE->cursorExcludedForUpdate = TRUE;
+            if (!pVMWARE->cursorExcludedForUpdate) {
+                PRE_OP_HIDE_CURSOR();
+                pVMWARE->cursorExcludedForUpdate = TRUE;
+            }
 	    break;
         }
         boxPtr++;
@@ -1138,6 +1342,71 @@ VMWARELoadPalette(ScrnInfoPtr pScrn, int numColors, int* indices,
     VmwareLog(("Palette loading done\n"));
 }
 
+
+DisplayModeRec *
+VMWAREAddDisplayMode(ScrnInfoPtr pScrn,
+                     const char *name,
+                     int width,
+                     int height)
+{
+   DisplayModeRec *mode;
+
+   mode = xalloc(sizeof(DisplayModeRec));
+
+   mode->name = xalloc(strlen(name) + 1);
+   strcpy(mode->name, name);
+   mode->status = MODE_OK;
+   mode->type = M_T_DEFAULT;
+   mode->HDisplay = width;
+   mode->VDisplay = height;
+
+   mode->next = pScrn->modes;
+   mode->prev = pScrn->modes->prev;
+   pScrn->modes->prev->next = mode;
+   pScrn->modes->prev = mode;
+
+   return mode;
+}
+
+
+#if VMWARE_DRIVER_FUNC
+static Bool
+VMWareDriverFunc(ScrnInfoPtr pScrn,
+                 xorgDriverFuncOp op,
+                 pointer data)
+{
+   CARD32 *flag;
+   xorgRRModeMM *modemm;
+
+   switch (op) {
+   case GET_REQUIRED_HW_INTERFACES:
+      flag = (CARD32 *)data;
+
+      if (flag) {
+         *flag = HW_IO | HW_MMIO;
+      }
+      return TRUE;
+   case RR_GET_MODE_MM:
+      modemm = (xorgRRModeMM *)data;
+
+      /*
+       * Because changing the resolution of the guest is usually changing the size
+       * of a window on the host desktop, the real physical DPI will not change. To
+       * keep the guest in sync, we scale the 'physical' screen dimensions to
+       * keep the DPI constant.
+       */
+      if (modemm && modemm->mode) {
+         modemm->mmWidth *= modemm->mode->HDisplay / (double)(modemm->virtX);
+         modemm->mmHeight *= modemm->mode->VDisplay / (double)(modemm->virtY);
+      }
+      return TRUE;
+   default:
+      return FALSE;
+   }
+}
+#endif
+
+
 static Bool
 VMWAREScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
 {
@@ -1160,9 +1429,7 @@ VMWAREScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
     VMWAREInitFIFO(pScrn);
 
     /* Initialise the first mode */
-    VMWAREModeInit(pScrn, pScrn->currentMode);
-
-    vmwareSendSVGACmdPitchLock(pVMWARE, pVMWARE->fbPitch);
+    VMWAREModeInit(pScrn, pScrn->currentMode, FALSE);
 
     /* Set the viewport if supported */
     VMWAREAdjustFrame(scrnIndex, pScrn->frameX0, pScrn->frameY0, 0);
@@ -1269,17 +1536,6 @@ VMWAREScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
         vmwareCursorHookWrappers(pScreen);
     }
 
-    /*
-     * Initialize acceleration.
-     */
-    if (!pVMWARE->noAccel) {
-        if (!vmwareXAAScreenInit(pScreen)) {
-            xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
-                       "XAA initialization failed -- running unaccelerated!\n");
-            pVMWARE->noAccel = TRUE;
-        }
-    }
-
     /*
      * If backing store is to be supported (as is usually the case),
      * initialise it.
@@ -1319,6 +1575,40 @@ VMWAREScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
         return FALSE;
     }
 
+    /*
+     * We explictly add a set of default modes because the X server will
+     * not include modes larger than the initial one.
+     */
+   {
+      unsigned int i;
+      unsigned int numModes = sizeof (VMWAREDefaultModes) / sizeof *(VMWAREDefaultModes);
+      char name[10];
+      for (i = 0; i < numModes; i++) {
+         const VMWAREDefaultMode *mode = &VMWAREDefaultModes[i];
+
+         /* Only modes that fit the hardware maximums should be added. */
+         if (mode->width <= pVMWARE->maxWidth && mode->height <= pVMWARE->maxHeight) {
+            snprintf(name, 10, "%dx%d", mode->width, mode->height);
+            VMWAREAddDisplayMode(pScrn, name, mode->width, mode->height);
+         }
+      }
+
+      /* Add the hardware maximums as a mode. */
+      snprintf(name, 10, "%dx%d", pVMWARE->maxWidth, pVMWARE->maxHeight);
+      VMWAREAddDisplayMode(pScrn, name, pVMWARE->maxWidth, pVMWARE->maxHeight);
+   }
+
+    /*
+     * We will lazily add the dynamic modes as the are needed when new
+     * modes are requested through the control extension.
+     */
+    pVMWARE->dynMode1 = NULL;
+    pVMWARE->dynMode2 = NULL;
+
+#if VMWARE_DRIVER_FUNC
+    pScrn->DriverFunc = VMWareDriverFunc;
+#endif
+
     /* Report any unused options (only for the first generation) */
     if (serverGeneration == 1) {
         xf86ShowUnusedOptions(pScrn->scrnIndex, pScrn->options);
@@ -1331,7 +1621,7 @@ VMWAREScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
 static Bool
 VMWARESwitchMode(int scrnIndex, DisplayModePtr mode, int flags)
 {
-    return VMWAREModeInit(xf86Screens[scrnIndex], mode);
+    return VMWAREModeInit(xf86Screens[scrnIndex], mode, TRUE);
 }
 
 static Bool
@@ -1344,9 +1634,7 @@ VMWAREEnterVT(int scrnIndex, int flags)
         VMWAREInitFIFO(pScrn);
     }
 
-    vmwareSendSVGACmdPitchLock(pVMWARE, pVMWARE->fbPitch);
-
-    return VMWAREModeInit(pScrn, pScrn->currentMode);
+    return VMWAREModeInit(pScrn, pScrn->currentMode, TRUE);
 }
 
 static void
@@ -1355,8 +1643,6 @@ VMWARELeaveVT(int scrnIndex, int flags)
     ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
     VMWAREPtr pVMWARE = VMWAREPTR(pScrn);
 
-    vmwareSendSVGACmdPitchLock(pVMWARE, 0);
-
     VMWARERestore(pScrn);
 }
 
@@ -1416,7 +1702,7 @@ VMWAREProbe(DriverPtr drv, int flags)
                                             NULL, NULL);
                 if (pScrn) {
                     VmwareLog(("And even configuration suceeded\n"));
-                    pScrn->driverVersion = VERSION;
+                    pScrn->driverVersion = VMWARE_DRIVER_VERSION;
                     pScrn->driverName = VMWARE_DRIVER_NAME;
                     pScrn->name = VMWARE_NAME;
                     pScrn->Probe = VMWAREProbe;
@@ -1436,14 +1722,18 @@ VMWAREProbe(DriverPtr drv, int flags)
     return foundScreen;
 }
 
+
 _X_EXPORT DriverRec VMWARE = {
-    VERSION,
+    VMWARE_DRIVER_VERSION,
     VMWARE_DRIVER_NAME,
     VMWAREIdentify,
     VMWAREProbe,
     VMWAREAvailableOptions,
     NULL,
-    0
+    0,
+#if VMWARE_DRIVER_FUNC
+    VMWareDriverFunc,
+#endif
 };
 
 #ifdef XFree86LOADER
@@ -1462,10 +1752,10 @@ vmwareSetup(pointer module, pointer opts, int *errmaj, int *errmin)
 
     if (!setupDone) {
         setupDone = TRUE;
-        xf86AddDriver(&VMWARE, module, 0);
+        xf86AddDriver(&VMWARE, module, VMWARE_DRIVER_FUNC);
 
         LoaderRefSymLists(vgahwSymbols, fbSymbols, ramdacSymbols,
-                          shadowfbSymbols, vmwareXaaSymbols, NULL);
+                          shadowfbSymbols, NULL);
 
         return (pointer)1;
     }
